// https://github.com/ros-planning/moveit2_tutorials/blob/main/doc/examples/moveit_cpp/src/moveit_cpp_tutorial.cpp

#include <rclcpp/rclcpp.hpp>
#include <string>
#include <thread>

// MoveitCpp
#include <moveit/moveit_cpp/moveit_cpp.h>
#include <moveit/moveit_cpp/planning_component.h>

#include "geometry_msgs/msg/pose_stamped.hpp"

const std::string PLANNING_GROUP = "ar3_arm";

class ScanNPlan : public rclcpp::Node {
public:
    ScanNPlan() : Node("scan_n_plan") {
        // create the node and start it
        // nh_ = std::make_shared<Node>(reinterpret_cast<Node *>(this));
	//nh_ = shared_from_this();

        RCLCPP_INFO(this->get_logger(), "SNP START");

        // create subscriber
        pose_sub_ = this->create_subscription<geometry_msgs::msg::PoseStamped>(
            "move_pose",
            rclcpp::QoS(1),
            std::bind(&ScanNPlan::moveCallback, this, std::placeholders::_1)
        );
        RCLCPP_INFO(this->get_logger(), "SUB EXIST");
    }

private:
    void moveCallback(geometry_msgs::msg::PoseStamped::SharedPtr p) {
	    (void) p;
    	RCLCPP_INFO(this->get_logger(), "GOT MSG");
        auto moveit_cpp_ptr = std::make_shared<moveit_cpp::MoveItCpp>(shared_from_this());
    	RCLCPP_INFO(this->get_logger(), "GOT MOVEIT");

        //auto psm = std::make_shared<planning_scene_monitor::PlanningSceneMonitor>("robot_description");
        //psm->startSceneMonitor();
        moveit_cpp_ptr->getPlanningSceneMonitor()->providePlanningSceneService();


        // planning_scene_monitor::LockedPlanningSceneRW scene{moveit_cpp_ptr->getPlanningSceneMonitor()};
    	RCLCPP_INFO(this->get_logger(), "NEW PLANNING");

        auto planning_components = std::make_shared<moveit_cpp::PlanningComponent>(PLANNING_GROUP, moveit_cpp_ptr);
    	RCLCPP_INFO(this->get_logger(), "GET MODEL");
        auto robot_model_ptr = moveit_cpp_ptr->getRobotModel();
    	RCLCPP_INFO(this->get_logger(), "GET START STATE");
        auto robot_start_state = planning_components->getStartState();

    	RCLCPP_INFO(this->get_logger(), "SETSTART TO CUR");
        planning_components->setStartStateToCurrentState();

        // set the goal to the request pose
    	RCLCPP_INFO(this->get_logger(), "SET GOAL");
        planning_components->setGoal(*p, "base_link");

        // actually plan
    	RCLCPP_INFO(this->get_logger(), "DO PLAN");
        auto plan_solution = planning_components->plan();

        // plan is valid
        if (plan_solution) {
            planning_components->execute();
        }
    }
private:
    rclcpp::Subscription<geometry_msgs::msg::PoseStamped>::SharedPtr pose_sub_;

    //please
    //rclcpp::Node::SharedPtr nh_;
    // moveit_cpp::MoveItCpp::SharedPtr moveit_cpp_ptr_;
    // rclcpp::executors::SingleThreadedExecutor::SharedPtr executor_;
    // std::thread thread_executor_spin_;
};

int main(int argc, char* argv[]) {
    rclcpp::init(argc, argv);

    auto node = std::make_shared<ScanNPlan>();
    RCLCPP_INFO(node->get_logger(), "MoveGroup node started.");

    rclcpp::spin(node);

    // spin the node
    // executor_ = std::make_shared<rclcpp::executors::SingleThreadedExecutor>();
    // executor_->add_node(nh_);

    RCLCPP_INFO(node->get_logger(), "Terminating MoveGroup");
    rclcpp::shutdown();
    return 0;
}

